@@ -19,8 +19,13 @@ import hashlib
 import traceback
 import threading
 import time
-import collections
 import types
+import collections
+import enum
+if sys.version_info >= (3, 4):
+    from collections.abc import KeysView, ValuesView, ItemsView
+else:
+    from collections import KeysView, ValuesView, ItemsView
 
 if sys.version_info < (2, 7):
     import unittest2 as unittest
@@ -77,9 +82,9 @@ class TestDeserialize(unittest.TestCase):
         self.assertEqual(set([1, 2, 3]), v)
 
     def test_unicode_escapes(self):
-        v = serpent.loads(b"'\u20ac'")
+        v = serpent.loads(b"'\\u20ac'")
         self.assertEqual(u"\u20ac", v)
-        v = serpent.loads(b"'\U00022001'")
+        v = serpent.loads(b"'\\U00022001'")
         self.assertEqual(u"\U00022001", v)
 
     def test_input_types(self):
@@ -409,6 +414,16 @@ class TestBasics(unittest.TestCase):
             self.assertEqual(ord("{"), data[0])
             self.assertEqual(ord("}"), data[-1])
 
+    def test_dict_unicode(self):
+        data = {"key": unicode("value")}
+        ser = serpent.dumps(data)
+        data2 = serpent.loads(ser)
+        self.assertEqual(unicode("value"), data2["key"])
+        data = {unicode("key"): 123}
+        ser = serpent.dumps(data)
+        data2 = serpent.loads(ser)
+        self.assertEqual(123, data2[unicode("key")])
+
     def test_dict_iters(self):
         data = {"john": 22, "sophie": 34, "bob": 26}
         ser = serpent.loads(serpent.dumps(data.keys()))
@@ -490,6 +505,14 @@ class TestBasics(unittest.TestCase):
         data = strip_header(ser)
         self.assertTrue(data == b"(42,'Sally')" or data == b"('Sally',42)")    # must output a tuple instead of a set-literal
 
+        # unicode elements
+        data = set([unicode("text1"), unicode("text2")])
+        ser = serpent.dumps(data)
+        data2 = serpent.loads(ser)
+        self.assertEqual(2, len(data2))
+        self.assertIn(unicode("text1"), data2)
+        self.assertIn(unicode("text2"), data2)
+
     def test_bytes(self):
         if sys.version_info >= (3, 0):
             ser = serpent.dumps(bytes(b"abcdef"))
@@ -613,19 +636,23 @@ class TestBasics(unittest.TestCase):
             serpent.dumps({1, 2, 3, pp})     # can only serialize simple typles as set elements (hashable)
         self.assertTrue("hashable type" in str(x.exception))
 
-    @unittest.skipIf(sys.version_info < (3, 4), reason="python 3.4 introduced enums")
     def test_enum_hashable(self):
-        import enum
         class Color(enum.Enum):
             RED = 1
             GREEN = 2
             BLUE = 3
         data = serpent.dumps({"abc", Color.RED, Color.GREEN, Color.BLUE})
         orig = serpent.loads(data)
-        self.assertEqual({"abc", 1, 2, 3}, orig)
+        if sys.version_info < (3, 4):
+            self.assertEqual([1, 2, 3, u"abc"], sorted(orig))
+        else:
+            self.assertEqual({"abc", 1, 2, 3}, orig)
         data = serpent.dumps({"abc": 1, Color.RED: 1, Color.GREEN: 1, Color.BLUE: 1})
         orig = serpent.loads(data)
-        self.assertEqual({"abc": 1, 1: 1, 2: 1, 3: 1}, orig)
+        if sys.version_info < (3, 4):
+            self.assertEqual({u"abc": 1, 1: 1, 2: 1, 3: 1}, orig)
+        else:
+            self.assertEqual({"abc": 1, 1: 1, 2: 1, 3: 1}, orig)
 
     def test_array(self):
         ser = serpent.dumps(array.array('u', unicode("unicode")))
@@ -710,16 +737,19 @@ class TestBasics(unittest.TestCase):
         v = serpent.loads(serpent.dumps((-98765432123456.12345678987656e+44 -665544332211.9998877665544e+33j)))
         self.assertEqual((-98765432123456.12345678987656e+44 -665544332211.9998877665544e+33j), v)
 
-    @unittest.skipIf(sys.version_info < (3, 4), "needs python 3.4 to test enum type")
     def test_enums(self):
-        import enum
         class Animal(enum.Enum):
             BEE = 1
             CAT = 2
             DOG = 3
         v = serpent.loads(serpent.dumps(Animal.CAT))
         self.assertEqual(2, v)
-        Animal2 = enum.Enum("Animals2", "BEE CAT DOG HORSE RABBIT")
+        class Animal2(enum.Enum):
+            BEE = 1
+            CAT = 2
+            DOG = 3
+            HORSE = 4
+            RABBIT = 5
         v = serpent.loads(serpent.dumps(Animal2.HORSE))
         self.assertEqual(4, v)
 
@@ -918,8 +948,11 @@ class Something(object):
     def __getstate__(self):
         return ("bogus", "state")
 
+
 class BaseClass(object):
     pass
+
+
 class SubClass(BaseClass):
     pass
 
@@ -978,14 +1011,12 @@ class TestCustomClasses(unittest.TestCase):
         serpent.register_class(BaseClass, lambda: None)
         serpent.register_class(SubClass, lambda: None)
         classes = list(serpent._special_classes_registry)
-        self.assertEqual(collections.KeysView, classes.pop(0))
-        self.assertEqual(collections.ValuesView, classes.pop(0))
-        self.assertEqual(collections.ItemsView, classes.pop(0))
+        self.assertEqual(KeysView, classes.pop(0))
+        self.assertEqual(ValuesView, classes.pop(0))
+        self.assertEqual(ItemsView, classes.pop(0))
         if sys.version_info >= (2, 7):
             self.assertEqual(collections.OrderedDict, classes.pop(0))
-        if sys.version_info >= (3, 4):
-            import enum
-            self.assertEqual(enum.Enum, classes.pop(0))
+        self.assertEqual(enum.Enum, classes.pop(0))
         self.assertEqual(BaseClass, classes.pop(0))
         self.assertEqual(SubClass, classes.pop(0))
         self.assertEqual(0, len(classes))
